@@ -18,6 +18,8 @@
 #include "nanopolish_call_methylation.h"
 #include "nanopolish_scorereads.h"
 #include "nanopolish_phase_reads.h"
+#include "nanopolish_vcf2fasta.h"
+#include "nanopolish_polya_estimator.h"
 #include "nanopolish_train_poremodel_from_basecalls.h"
 
 int print_usage(int argc, char **argv);
@@ -34,7 +36,9 @@ static std::map< std::string, std::function<int(int, char**)> > programs = {
     {"variants",    call_variants_main},
     {"methyltrain", methyltrain_main},
     {"scorereads",  scorereads_main} ,
-    {"phase-reads",  phase_reads_main} ,
+    {"phase-reads", phase_reads_main} ,
+    {"vcf2fasta",   vcf2fasta_main} ,
+    {"polya",  polya_main} ,
     {"call-methylation",  call_methylation_main}
 };
 
